var map, markers; // map and marker object
var spinnerMarker; // The marker used for the spinning animation

var permaZoom, permaLon, permaLat, permaLayers; // variables for the Permalink parameters

// Default position and zoom on Leipzig
var defLat = 51.3333; 
var defLon = 12.38333;
var defZoom = 5;

var defIconHeight = 32;
var defIconWidth = 32;

var popup, currentLat, currentLon; // to be used within moveMarker()


/**
 * Function: MapView
 * Creates a MapView object usingConverts longitude to mercator projection longitude
 * @param {String} HTMLlayer The id of the  HTML-element that will contain the map
 * @param {String} abstraction The abstraction library to be used. Valid: openlayers (TODO: mapstraction). Defaults to: openlayers
 * @param {String} provider The default map provider to be used when initializing the map object
 * @author OntoWiki MapPlugin Team (Claudius Henrichs)
 * @version 1.0
 * @constructor
 */
function MapView(HTMLlayer, abstraction, provider) {
    // Create a new map object in the given HTML layer
    map = new OpenLayers.Map( HTMLlayer);
    
    // Add controls for switching layers
    map.addControl( new OpenLayers.Control.LayerSwitcher());
	
    // Adds a permalink to the current map view in the lower right corner
    map.addControl( new OpenLayers.Control.Permalink() );
    
    // Add map scale line to lower left corner
    //map.addControl( new OpenLayers.Control.ScaleLine() );
	
    // Adds current mouse cursor position output latitude and longitude in the lower right corner
    //map.addControl(new OpenLayers.Control.MousePosition());

	// Adds map overview to the lower right corner
    // map.addControl(new OpenLayers.Control.OverviewMap());
	
    // Create a set of Google Map layers for physical, street, hybrid and satellite view
    var gphy = new OpenLayers.Layer.Google(
            "Google Physical",
            {type: G_PHYSICAL_MAP}
            );
    var gmap = new OpenLayers.Layer.Google(
            "Google Streets" // the default
            );
    var ghyb = new OpenLayers.Layer.Google(
            "Google Hybrid",
            {type: G_HYBRID_MAP}
            );
    var gsat = new OpenLayers.Layer.Google(
            "Google Satellite",
            {type: G_SATELLITE_MAP}
            );

    // Create a Yahoo map layer
    var yahooLayer = new OpenLayers.Layer.Yahoo( "Yahoo");

    // Create a WMS layer
    var wms_layer = new OpenLayers.Layer.WMS( "OpenLayers WMS",
            "http://labs.metacarta.com/wms/vmap0", {layers: 'basic'} );

    // Create an OpenStreetMap Mapnik layer in the correct projection. See www.openstreetmap.org for details
    var mapnik_layer = new OpenLayers.Layer.WMS( "OSM Mapnik", 
            [
            "http://t1.hypercube.telascience.org/tiles?",
            "http://t2.hypercube.telascience.org/tiles?",
            "http://t3.hypercube.telascience.org/tiles?",
            "http://t4.hypercube.telascience.org/tiles?"
            ], 
            {layers: 'osm-4326', format: 'image/png' } );

    // This Osmarender layer does not work as it is using EPSG:900913 projection
    //var TilesAtHome_layer    = new OpenLayers.Layer.OSM.Osmarender("Osmarender");

    // Adds the layers to the map-object called 'map'
    map.addLayers( [gphy, gmap, ghyb, gsat, mapnik_layer, wms_layer, yahooLayer]);
    
    markers = new OpenLayers.Layer.Markers( "Markers" );
    map.addLayer(markers);
    
    // Centers the map on Leipzig if no permalink is called
    // else the permalink is shown
    if (permaLon == 'null'){
        permaLon = defLon;
    }
    if (permaLat == 'null'){
        permaLat = defLat;
    }
    if (permaZoom == 'null'){
        permaZoom = defZoom;
    }
	
	// Add  shrink/enlarge controls to the Openlayers map object
    $jq("#map_OpenLayers_ViewPort").append("<div id='MapPluginControls' class='olControlSelect' style='position: absolute; right: 0px; bottom: 24px; z-index: 1006;'></div>");
    $jq("#MapPluginControls").append("<div id='ShrinkMap' style='position: absolute; right: 0px; bottom: 18px; z-index: 1006;'><img id='ShrinkMapButton' style='position: relative; width: 18px; height: 18px;' src='../../plugins/MapPlugin/media/images/shrink-mini.png'/></div>");
	$jq("#MapPluginControls").append("<div id='EnlargeMap' style='position: absolute; right: 0px; bottom: 0px; z-index: 1006;'><img id='EnlargeMapButton' style='position: relative; width: 18px; height: 18px;' src='../../plugins/MapPlugin/media/images/enlarge-mini.png'/></div>");

        
    // ...and add a click-action for the functions to shrink/enlarge the map
    $jq('#ShrinkMapButton').click(function(){
      if ($jq('#map').height() > 250) {$jq('#map').height($jq('#map').height()-100);}
    });
    $jq('#EnlargeMapButton').click(function(){
      $jq('#map').height($jq('#map').height()+100);
    }); 
    

    // Observe the map and notify ... when the map was moved
    map.events.register("moveend", map, updateBound);
    
    // If permalink has been given center on it.
    if (permaLayers != 'null'){
        map.setCenter(new OpenLayers.LonLat(permaLon, permaLat), permaZoom);
    } else {
        // Issue initial JSON marker call with empty view area
        if( uri.indexOf( "?" ) < 0 ) {
    		var uriConcatChar = "?";
    	}
    	else {
    		var uriConcatChar = "&";
    	}
        
        var clusterOn;
        if ($jq('#clusterOn:checked').val() != null) {
            clusterOn = 1;
        } else {
            clusterOn = 0;
        }
        map.setCenter(new OpenLayers.LonLat(defLon, defLat), defZoom);
        //getAndDisplayMarkers("../../service/map/" + uri + uriConcatChar + "clusterOn=" + clusterOn + "&clustGridCount=" + $jq('#clustGridCount').attr("value") + "&clustMaxMarkers=" + $jq('#clustMaxMarkers').attr("value") );
    }    
}

/**
 * Function: setPermaParams
 * Sets the zoomlevel, latitude, longitude and the layer from a permalink (see Map template)
 * @param {integer} permazoom Zoomlevel of the permalink
 * @param {integer} permalon  longitude of the permalink
 * @param {integer} permalat latitude of the permalink
 * @param {integer} permalayer Zoomlevel of the permalink
 */
function setPermaParams(permazoom, permalon, permalat, permalayers){
    this.permaZoom = permazoom;
    this.permaLon = permalon;
    this.permaLat = permalat;
    this.permaLayers = permalayers;
}

/**
 * Function: getAndDisplayMarkers
 * Remove all current markers, get new markers via JSON and display them using JQuery + the abstraction provider
 * @param {String} JSON_URL URL to get the JSON object from. This might contain URI parameters
  * @param {String} abstraction The abstraction library to be used. Valid: openlayers (TODO: mapstraction). Defaults to: openlayers
 */
function getAndDisplayMarkers(JSON_URL, abstraction){    
    //Close any Popup that might still be open
    closeMapPopup();
       
    //Remove the current marker layer
    if (markers) {
        markers.destroy();
    }
    
	// Welcome the Phoenix! Create a brand new marker-layer and add it to the map
	markers = new OpenLayers.Layer.Markers( "Markers" );
    map.addLayer(markers);    
    
    // Add a fake marker centered on the map with the spinning icon as long as the JSON is in progress
    spinnerMarker = addMarkerToMap(map.getCenter(), "../../themes/default/images/spinner.gif", null, 16, 16);

	// getJSON object from URL and execute callback function if successfull - http://docs.jquery.com/Ajax/jQuery.getJSON#urldatacallback
    $jq.getJSON( JSON_URL,
        function( data ) {
            if (data.marker != null) {
                // Add a marker for each marker returned in the JSON object ($jq) if the marker-array is not empty
                $jq.each( data.marker, function( i, item ) {
                    // Some debug output
                    //log ('Adding marker ' + item.label + ' at ' + item.lat + ', ' + item.long + ' with icon ' + item.icon);
                    
                    // Adds the marker to the marker layer using the magical addMarkerToMap (see below)
                    if (item.label == "cluster") {
                        // If the marker is a cluster, concatenate the URIs for the bubble
                        var clustertext = "";
                        $jq.each(item.markers, function(m, marker){
                            clustertext += "<a href='../../resource/view/?r=" + marker.uri + "'>" + marker.label + "</a><br />";
                        });
                        addMarkerToMap(new OpenLayers.LonLat(item.long, item.lat), "../../plugins/MapPlugin/media/images/marker-clustered.png", clustertext, 21, 25);
                    } else if (!item.icon) {
                        addMarkerToMap(new OpenLayers.LonLat(item.long, item.lat), "../../plugins/MapPlugin/media/images/marker.png", "<a href='../../resource/view/?r=" + item.uri + "'>" + item.label + "</a><br /><a href='#' onclick='setRouteTarget(\"" + item.lat + ", " + item.long + "\");'>Route to</a> | <a href='#' onclick='moveMarker(\""+item.uri+ "\", "+item.lat+", "+item.long+");closeMapPopup();'>Move</a>", 21, 25, -10, -25);
                    } else {
                        addMarkerToMap(new OpenLayers.LonLat(item.long, item.lat), item.icon, "<img src='" + item.icon +"' alt='Ressource-Icon' /><br /><a href='../../resource/view/?r=" + item.uri + "'>" + item.label + "</a><br /><a href='#' onclick='setRouteTarget(\"" + item.lat + ", " + item.long + "\");'>Route to</a> | <a href='#' onclick='moveMarker(\""+item.uri+ "\", "+item.lat+", "+item.long+");closeMapPopup();'>Move</a>", 21, 25);
                    }
                    });
            }
            bestviewarea = data.area;
            
            if (map.getExtent() == null) {
                log ('Best view area!');
                // Zoom to best view area if now view area is set yet
                setBestViewArea(data.area);
            }
            
            // Remove the spinner marker from the map once all marker object have been parsed
            if (spinnerMarker) {
                markers.removeMarker(spinnerMarker);
            }
            
            // Zoom out to show the world map. TODO: only the map view for the just loaded markers
            //map.zoomToMaxExtent();
            }
            );
};


/**
 * Sets the given LatLon pair as target for routing and shows the routing panel if hidden
 * @returns true
 */
function setBestViewArea(bestviewarea) {
   log ('bestviewarea: ' + bestviewarea);
    // Observe the map and notify ... when the map was moved
    map.events.unregister( "moveend", map, updateBound);
    
    bounds = new OpenLayers.Bounds();
    //bounds.extend( new OpenLayers.LonLat( bestviewarea[3], bestviewarea[2] ) );
    //bounds.extend( new OpenLayers.LonLat( bestviewarea[1], bestviewarea[0] ) );
    bounds.extend( new OpenLayers.LonLat( 10, 10 ) );
    bounds.extend( new OpenLayers.LonLat( -10, -10 ) );
    
    //map.zoomToExtent( bounds );
    
    //map.setCenter(new OpenLayers.LonLat(bestviewarea[3], bestviewarea[2]), permaZoom);
    
    // Observe the map and notify ... when the map was moved
    map.events.register( "moveend", map, updateBound);
}


/**
 * Sets the given LatLon pair as target for routing and shows the routing panel if hidden
 * @param {String} LongLatString Comma-seperated string with longitude and latitude
 * @returns true
 */
function setRouteTarget(LongLatString) {
    // Paste the LatLonString in the  destination address
    $jq('#route_daddr').attr("value", LongLatString);
    
	// Unhide Routingwidget - TODO: Check if the routing Box is already maximized
    //toggleWindow($jq('#routing'));
    
    // Select current entry in routing start and set focus in that field
    $jq('#route_saddr').select();   
    $jq('#route_saddr').focus();
	
    closeMapPopup();
    
	return true;
}


/**
 * Move an existing marker to a new position and draw an edge between the old
 * and new marker position.
 * @param uri identifies the ressource
 * @param lat latitude of the current markerposition
 * @param lon longitude of the current markerposition
 */
function moveMarker(uri, lat, lon) {
    currentLat = lat;
    currentLon = lon;

    // Adds current mouse cursor position output latitude and longitude in the lower right corner
    map.addControl(new OpenLayers.Control.MousePosition());

    // Observe the map and notify ... when the map has been moved
    map.events.register("click", map, setNewPosition);
}


/**
 * Move an existing marker to a new position and draw an edge between the old
 * and new marker position.
 * @param uri identifies the ressource
 */
function setNewPosition(evt){

    // select the click-point
    var ll = map.getLonLatFromViewPortPx(new OpenLayers.Pixel(evt.xy.x, evt.xy.y) );

    marker = addMarkerToMap(new OpenLayers.LonLat(ll.lon, ll.lat), "../../plugins/MapPlugin/media/images/marker-new.png", "Move Target", 21, 25, -10, -25 );
   
    openMapPopup(marker, "Distance:<br />");
   
    map.events.unregister("click", map, setNewPosition);
    
    // Add a new vector object layer to the map...
    var vectors = new OpenLayers.Layer.Vector("Vectors");
    map.addLayer(vectors);

    //... so that we can draw a line between the old and new position
    var style_blue = OpenLayers.Util.extend({}, OpenLayers.Feature.Vector.style['default']);
    style_blue.strokeColor = "blue"; 
    style_blue.fillColor = "blue"; 
    
    // A stylish green style for a stylish connection line
    var style_red = {
        strokeColor: "#FF3333",
        strokeOpacity: 1,
        strokeWidth: 3,
        pointRadius: 6,
        pointerEvents: "visiblePainted"
    };
    
    // Create a point  at the move target
    var point = new OpenLayers.Geometry.Point( ll.lon, ll.lat);
    //pointFeature = new OpenLayers.Feature.Vector(point, null, style_blue);
    
    // create a line feature from  the old to the new marker position
    var pointList = [];
    pointList.push(point);
    pointList.push( new OpenLayers.Geometry.Point(currentLon, currentLat));

    lineFeature = new OpenLayers.Feature.Vector(
        new OpenLayers.Geometry.LineString(pointList), null, style_red);

    vectors.addFeatures([lineFeature]);
    
    // Remove the MousePosition-control once the move target has been set
    //map.removeControl(OpenLayers.Control.MousePosition());
}


/**
 * Function: updateBound
 * If the map has been moved issue a new JSON GET  to get the markers for the new map viewport
 */
function updateBound()
{
	// Check if the URI ontains another set of GETS and choose concatenation symbol accordingly
	if( uri.indexOf( "?" ) < 0 ) {
		var uriConcatChar = "?";
	}
	else {
		var uriConcatChar = "&";
	}
    
    var clusterOn;
    if ($jq('#clusterOn:checked').val() != null) {
        clusterOn = 1;
    } else {
        clusterOn = 0;
    }
    // Issue JSON call width map extents
    getAndDisplayMarkers("../../service/map/" + uri + uriConcatChar + "viewArea=" +  map.getExtent().toArray()[3] + "," +  map.getExtent().toArray()[2] + "," +  map.getExtent().toArray()[1] + "," +  map.getExtent().toArray()[0] + "&clusterOn=" + clusterOn + "&clustGridCount=" + $jq('#clustGridCount').attr("value") + "&clustMaxMarkers=" + $jq('#clustMaxMarkers').attr("value") );
}

/**
 * Function: addMarkerToMap
 * Add a new marker to the markers layer given the following lonlat, 
 *     popupClass, and popup contents HTML. Also allow specifying 
 *     whether or not to give the popup a close box.
 * 
 * Parameters:
 * @param   {<OpenLayers.LonLat>} position -Where to place the marker
 * @param  {String} icon The relative (absolute?) path to the icon image ressource
 * @param  {String} description What to put in the popup
 * @param  {String} iconWidth  (optional) icon width in pixels (defaults to 32)
 * @param  {String} iconHeight  (optional) icon height in pixels (defaults to 32) 
 * @param  {String} horizontaloffset (optional) horizontal offset in pixels (defaults to center width)
 * @param  {String} verticaloffset (optional) vertical offset in pixels (defaults to bottom)
 */
function addMarkerToMap(position, icon, description, iconWidth, iconHeight, horizontaloffset, verticaloffset) {    
    //Default icon width and height if parameters are empty
    iconHeight = iconHeight || defIconHeight; 
    iconWidth = iconWidth || defIconWidth;
    
    // Calculate default icon offset if parameters are empty
    var size = new OpenLayers.Size(iconWidth, iconHeight);    
    horizontaloffset = horizontaloffset || -(size.w/2);
    verticaloffset = verticaloffset || -size.h; 
    var offset = new OpenLayers.Pixel(horizontaloffset, verticaloffset);
     
    // Create the marker and add it to the "markers"-layer
    var marker = new OpenLayers.Marker(position, new OpenLayers.Icon(icon, size, offset) );
    markers.addMarker(marker);
    
    // If there is  a description create a popup containing it
    if (description) {
        marker.events.register("click", marker, function() { openMapPopup(marker, description) });
    }

    return marker;
}

/**
 * Function: addMarkerToMap
 * Add a new marker to the markers layer given the following lonlat, 
 *     popupClass, and popup contents HTML. Also allow specifying 
 *     whether or not to give the popup a close box.
 * 
 * Parameters:
 * @param   {<OpenLayers.LonLat>} position -Where to place the marker
 * @param {<OpenLayers.Class>}  popupClass Which class of popup to bring up 
 *     when the marker is clicked.
 * @param  {String} icon The relative (absolute?) path to the icon image ressource
 * @param  {String} description What to put in the popup
 * @param  {String} iconWidth  (optional) icon width in pixels (defaults to 32)
 * @param  {String} iconHeight  (optional) icon height in pixels (defaults to 32) 
 * @param  {String} horizontaloffset (optional) horizontal offset in pixels (defaults to center width)
 * @param  {String} verticaloffset (optional) vertical offset in pixels (defaults to bottom)
 */
function openMapPopup(marker, description) {
   closeMapPopup();

   popup = new OpenLayers.Popup.AnchoredBubble("popup", marker.lonlat, null,
                                               description, marker.icon, true);
   popup.setBackgroundColor("#E3FFC5");
   popup.autoSize = true;
   map.addPopup(popup);

   return popup;
}

function closeMapPopup() {
   if (popup) {
      map.removePopup(popup);
      delete popup;
   }
}


/**
 * Function: log
 * A logging function for debugging purposes
 * @param {String} output The output string that should be added to the "debug console"
 */
function log(output) {
    document.getElementById('eventlog').innerHTML += output + '<br />';
}


/**
 * Function: is_array
 * Checks if the given variable is an array
 * @param {String} input The  JS-object to check
  * @returns true if object is an array
  */
function is_array(input) {
    return typeof(input)=='object'&&(input instanceof Array);
}

  
/**
 * Uses OntoWiki's default JQuery object $jq to define a startup function
 */
$jq(document).ready(function() {
    //Add a slider control
    //$jq("#gridcount_slider").slider();
    
    // Inits the map using OpenLayers, passing the HTML-div which should contain the map
    MapView('map', 'openlayers', 'openstreetmap');
    
    /*
    getAndDisplayMarkers("../../service/map/" + uri + uriConcatChar + "clusterOn=" + clusterOn + "&clustGridCount=" + $jq('#clustGridCount').attr("value") + "&clustMaxMarkers=" + $jq('#clustMaxMarkers').attr("value") );
    
    map.setCenter(new OpenLayers.LonLat(defLon, defLat), defZoom);*/
    });

    
// Hide Routingwidget - TODO: Construct the routing box minimized
//if (!$("*").index( $('#button button-windowminimize')[0] ))
toggleWindow($jq('#routing'));